package com.ozs.web.controller.accountmanagment;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.service.entity.*;
import com.ozs.service.entity.vo.*;
import com.ozs.service.mapper.MsgAlarmMapper;
import com.ozs.service.service.*;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysDictDataService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警信息表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@RestController
@RequestMapping("/service/msgAlarm")
public class MsgAlarmController extends BaseController {
    private static final Logger log = LoggerFactory.getLogger(MsgAlarmController.class);
    public static final String PATTERN = "(^K|^k)([0-9]+)\\+([0-9]{1,3})$";
    @Resource
    MsgAlarmService msgAlarmService;
    @Resource
    MsgAlarmMapper msgAlarmMapper;
    @Resource
    MsgAlarmFrequencyService msgAlarmFrequencyService;
    @Autowired
    BaseCameraManagementService baseCameraManagementService;
    @Autowired
    BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private MsgWebPushService msgWebPushService;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private DataScoreUtil dataScoreUtil;
    @Resource
    private BaseUserService baseUserService;

    /**
     * web更多消息-总数，已读数
     */
    @PostMapping("/messageMoreNum")
    @ApiOperation("更多消息--总数，已读数")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMoreNum(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        Map<String, Integer> messageMore = msgWebPushService.messageMoreNum(getUserId());
        return AjaxResult.success(messageMore);
    }

    /**
     * web更多消息
     */
    @PostMapping("/messageMore")
    @ApiOperation("更多消息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult messageMore(@RequestBody MsgAlarmVo msgAlarmVo) {
        // MsgAlarmVo msgAlarmVo1 = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        String userId = getUserId();
        if (!StringUtils.isEmpty(userId)){
            msgAlarmVo.setUserId(userId);
        }
        IPage<MsgAlarm> messageMore = msgWebPushService.messageMore(msgAlarmVo);
        return AjaxResult.success(messageMore);
    }

    /**
     * web报警未读消息个数+未读消息列表
     */
    @GetMapping("/webAlarm")
    @ApiOperation("app报警推送次数")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult webAlarmNum() {
        // MsgAlarmVo msgAlarm = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), new MsgAlarmVo());
        Map webAlarm = msgWebPushService.webAlarm(getUserId());
        return AjaxResult.success(webAlarm);
    }

    /**
     * web消息已读
     */
    @PostMapping("/read")
    @ApiOperation(value = "web消息已读")  
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult read(@RequestBody MsgWebPush msgWebPush) {
        if (StringUtils.isEmpty(msgWebPush) || StringUtils.isEmpty(msgWebPush.getId())) {
            return AjaxResult.error("主键ID不能为空！");
        }
        return toAjax(msgWebPushService.read(msgWebPush));
    }

    /**
     * 报警信息分页
     *
     * @param msgAlarmVo
     * @return
     */
    @ApiOperation(value = "报警信息分页")
    @PostMapping("/list")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult selectMsgAlarm(@RequestBody MsgAlarmVo msgAlarmVo) {
        msgAlarmVo = (MsgAlarmVo) dataScoreUtil.setDataScore(getUserId(), msgAlarmVo);
        if (!ObjectUtils.isEmpty(msgAlarmVo.getAlarmMiles())){
            if (!msgAlarmVo.getAlarmMiles().matches(PATTERN)) {
                return error("里程位置填写格式不正确!");
            }else {
                int miles = AppendUtils.stringSplit(msgAlarmVo.getAlarmMiles());
                msgAlarmVo.setAlarmMile(miles);
            }
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getBeginMiles())) {
            if (!msgAlarmVo.getBeginMiles().matches(PATTERN)) {
                return error("监控范围开始里程位置填写格式不正确");
            } else {
                int begin = AppendUtils.stringSplit(msgAlarmVo.getBeginMiles());
                msgAlarmVo.setBeginMile(begin);
            }
        }
        if (!ObjectUtils.isEmpty(msgAlarmVo.getEndMiles())) {
            if (!msgAlarmVo.getEndMiles().matches(PATTERN)) {
                return error("监控范围结束里程位置填写格式不正确");
            } else {
                int end = AppendUtils.stringSplit(msgAlarmVo.getEndMiles());
                msgAlarmVo.setEndMile(end);
            }
        }
        IPage<MsgAlarm> page = msgAlarmService.queryPage(msgAlarmVo);
        page.setTotal(page.getTotal());
        page.setCurrent(page.getCurrent());
        page.setPages(page.getPages());

        if (!ObjectUtils.isEmpty(page) && page.getRecords().size() > 0) {
            List<MsgAlarm> dto1 = page.getRecords().stream().map(o -> {
                String[] split = o.getImageUrl().split(";");
                ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
                o.setImageUrls(objects);
                LambdaQueryWrapper<BaseCameraManagement> cameraManagementLambdaQueryWrapper = new LambdaQueryWrapper<BaseCameraManagement>();
                if (!ObjectUtils.isEmpty(o.getCameraCode())) {
                    cameraManagementLambdaQueryWrapper.eq(BaseCameraManagement::getCameraCode, o.getCameraCode());
                }
                BaseCameraManagement cameraManagement = baseCameraManagementService.getOne(cameraManagementLambdaQueryWrapper);
                LambdaQueryWrapper<BaseRailwayManagement> queryWrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
                if (!ObjectUtils.isEmpty(cameraManagement.getRailwayCode())) {
                    queryWrapper.eq(BaseRailwayManagement::getRailwayCode, cameraManagement.getRailwayCode());
                }
                BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(queryWrapper);
                o.setRailwayName(baseRailwayManagement.getRailwayName());
                if (StringUtils.isNotEmpty(o.getReleasedBy())) {
                    BaseUser user = baseUserService.getUser(o.getReleasedBy());
                    o.setReleasedByName(user.getNickName());
                }
                String mils = AppendUtils.stringAppend(cameraManagement.getInstallMile());
                o.setAlarmMiles(mils);
                String end = AppendUtils.stringAppend(cameraManagement.getEndMile());
                o.setEndMile(end);
                String begin = AppendUtils.stringAppend(cameraManagement.getBeginMile());
                o.setBeginMile(begin);
                o.setAlarmCount((int) msgAlarmFrequencyService.count());
                return o;
            }).collect(Collectors.toList());
            page.setRecords(dto1);
        }
        return AjaxResult.success(page);
    }

    @GetMapping(value = "/getByAlarmId/{alarmId}")
    @ApiOperation("根据报警信息ID获取详细信息-app")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getByAlarmId(@PathVariable String alarmId) {
        MsgAlarm msgAlarm = msgAlarmService.getByAlarmId(alarmId);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = begin + "-" + end;
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据报警信息ID获取详细信息")
    @Log(title = "报警信息管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getInfo(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        if (!ObjectUtils.isEmpty(msgAlarm)) {
            LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
            if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
                lw.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
            }
            BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(lw);
            LambdaQueryWrapper<BaseRailwayManagement> wrapper = new LambdaQueryWrapper<BaseRailwayManagement>();
            if (!ObjectUtils.isEmpty(baseCameraManagement.getRailwayCode())) {
                wrapper.eq(BaseRailwayManagement::getRailwayCode, baseCameraManagement.getRailwayCode());
            }
            BaseRailwayManagement baseRailwayManagement = baseRailwayManagementService.getOne(wrapper);
            MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
            String[] split = msgAlarm.getImageUrl().split(";");
            ArrayList<String> objects = new ArrayList<>(Arrays.asList(split));
            msgAlarm.setImageUrls(objects);
            BeanUtils.copyProperties(msgAlarm, msgAlarmResp);
            msgAlarmResp.setRailwayName(baseRailwayManagement.getRailwayName());
            msgAlarmResp.setDeptId(baseCameraManagement.getDeptId());
            String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
            String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
            String endMile = "["+begin + "]-[" + end+"]";
            msgAlarmResp.setBeginMile(endMile);
            String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
            msgAlarmResp.setAlarmMile(mile);
            msgAlarmResp.setInstallLongitude(baseCameraManagement.getInstallLongitude() + "," + baseCameraManagement.getInstallLatitude());
            return AjaxResult.success(msgAlarmResp);
        } else {
            return success("无报警信息数据");
        }
    }

    @PutMapping(value = "/{id}")
    @ApiOperation("根据报警信息ID解除报警")
    @Log(title = "报警信息管理", businessType = BusinessType.UPDATE)
    public AjaxResult updateIsLock(@PathVariable Long id) {
        MsgAlarm msgAlarm = msgAlarmService.getById(id);
        msgAlarm.setIsLock(1);
        msgAlarm.setUpdateBy(getUserId());
        msgAlarm.setReleasedBy(getUserId());
        msgAlarm.setReleasedTime(new Date());
        return AjaxResult.success(msgAlarmService.updateById(msgAlarm));
    }

    @GetMapping(value = "/alarmHistoricalAlarm/{alarmId}")
    @ApiOperation("报警消息详情历史报警次数信息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult alarmHistoricalAlarm(@PathVariable Long alarmId) {
        QueryWrapper<MsgAlarmFrequency> wrapper = new QueryWrapper<>();
        wrapper.eq("alarm_id", alarmId);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarmFrequency> list = msgAlarmFrequencyService.list(wrapper);
        return AjaxResult.success(list);
    }

    @GetMapping(value = "/videoHistoricalAlarm/{cameraCode}")
    @ApiOperation("视频服务历史报警信息")
    @Log(title = "报警信息管理", businessType = BusinessType.SELECT)
    public AjaxResult videoHistoricalAlarm(@PathVariable String cameraCode) {
        QueryWrapper<MsgAlarm> wrapper = new QueryWrapper<>();
        wrapper.eq("camera_code", cameraCode);
        wrapper.orderByDesc("alarm_time");
        List<MsgAlarm> list = msgAlarmService.list(wrapper);
        return AjaxResult.success(list);
    }

    /**
     * web数据统计
     *
     * @param alarmStatisticResVo
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/dataStatistic")
    @ApiOperation(value = "web数据统计 报警数据详情")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "startMonth", value = "起始月份"),
            @ApiImplicitParam(paramType = "query", name = "endMonth", value = "结束月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public AjaxResult dataStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        Integer pageNum = alarmStatisticResVo.getPageNum().intValue();
        Integer pageSize = alarmStatisticResVo.getPageSize().intValue();
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        IPage<AlarmStatisticVo> page = msgAlarmService.list(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getStartMonth(),
                alarmStatisticResVo.getEndMonth(), alarmStatisticResVo.getAlarmType(), pageNum, pageSize, userId);
        return AjaxResult.success(page);
    }

    /**
     * web数据统计
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.OTHER)
    @PostMapping("/alarmMonthStatistic")
    @ApiOperation(value = "web数据统计 报警月统计数据")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "startMonth", value = "起始月份"),
            @ApiImplicitParam(paramType = "query", name = "endMonth", value = "结束月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public AjaxResult alarmMonthStatistic(@RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        ;
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        List<AlarmStatisticMonthDto> list = msgAlarmService.getCount(alarmStatisticResVo.getRailwayCode(), alarmStatisticResVo.getStartMonth(),
                alarmStatisticResVo.getEndMonth(), alarmStatisticResVo.getAlarmType(), userId);
        List<Integer> numbs = new ArrayList<>(Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12));
        List<Integer> list1 = new ArrayList<>();
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            list.forEach(l -> {
                list1.add(l.getMonthsta());
            });
        }
        AlarmStatisticMonthDto dto;
        for (int i = 0; i < numbs.size(); i++) {
            if (!list1.contains(numbs.get(i))) {
                dto = new AlarmStatisticMonthDto();
                dto.setMonthsta(numbs.get(i));
                dto.setFrequency(0);
                list.add(dto);
            }
        }
        if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
            Collections.sort(list, new Comparator<AlarmStatisticMonthDto>() {
                @Override
                public int compare(AlarmStatisticMonthDto o1, AlarmStatisticMonthDto o2) {

                    return o1.getMonthsta().compareTo(o2.getMonthsta());
                }
            });
        }
        return AjaxResult.success(list);
    }


    /**
     * web数据统计 数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatistic")
    @ApiOperation(value = "web数据统计-数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "startMonth", value = "起始月份"),
            @ApiImplicitParam(paramType = "query", name = "endMonth", value = "结束月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public void exportDataStatistic(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        try {
            List<AlarmStatisticDto> list = msgAlarmService.exportExcel(alarmStatisticResVo, userId);
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    String alarmType = l.getAlarmType();
                    String alarmTypeValue = dictDataService.selectDictLabel("sys_alarm_type", alarmType);
                    l.setAlarmTypeValue(alarmTypeValue);

                    String lineDir = l.getLineDir();
                    String lineDirValue = dictDataService.selectDictLabel("sys_line_dir", lineDir);
                    l.setLineDirValue(lineDirValue);
                    l.setAlarmMile(AppendUtils.stringAppend(Integer.valueOf(l.getAlarmMile())));
                });
            }

            ExcelUtil<AlarmStatisticDto> util = new ExcelUtil<>(AlarmStatisticDto.class);
            util.exportExcel(response, list, "报警数据详情");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
        }
    }

    /**
     * web数据统计 报警月统计数据导出
     *
     * @param
     * @return
     */
    @Log(title = "报警信息管理", businessType = BusinessType.EXPORT)
    @PostMapping("/exportDataStatisticMonth")
    @ApiOperation(value = "报警月统计数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "railwayCode", value = "线路编码"),
            @ApiImplicitParam(paramType = "query", name = "startMonth", value = "起始月份"),
            @ApiImplicitParam(paramType = "query", name = "endMonth", value = "结束月份"),
            @ApiImplicitParam(paramType = "query", name = "alarmType", value = "灾害类型"),
    })
    public void exportDataStatisticMonth(HttpServletResponse response, @RequestBody AlarmStatisticResVo alarmStatisticResVo) {
        SysUser user = getLoginUser().getUser();
        String userId = getUserId();
        if (user.isAdmin(user.getId())) {
            userId = "";
        }
        try {
            List<AlarmStatisticMonthDto> list = msgAlarmService.exportExcelMonth(alarmStatisticResVo, userId);
            List<Integer> numbs = new ArrayList<>(Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12));
            List<Integer> list1 = new ArrayList<>();
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                list.forEach(l -> {
                    list1.add(l.getMonthsta());
                });
            }
            AlarmStatisticMonthDto dto;
            for (int i = 0; i < numbs.size(); i++) {
                if (!list1.contains(numbs.get(i))) {
                    dto = new AlarmStatisticMonthDto();
                    dto.setMonthsta(numbs.get(i));
                    dto.setFrequency(0);
                    list.add(dto);
                }
            }
            if (!CollectionUtils.isEmpty(list) && Objects.nonNull(list.get(0))) {
                Collections.sort(list, new Comparator<AlarmStatisticMonthDto>() {
                    @Override
                    public int compare(AlarmStatisticMonthDto o1, AlarmStatisticMonthDto o2) {

                        return o1.getMonthsta().compareTo(o2.getMonthsta());
                    }
                });
            }
            ExcelUtil<AlarmStatisticMonthDto> util = new ExcelUtil<>(AlarmStatisticMonthDto.class);
            util.exportExcel(response, list, "报警月统计数据");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
        }
    }
}

